from django.conf import settings as global_settings
from django.contrib.auth.decorators import login_required
from django.shortcuts import redirect, render

from django_admin_otp import settings, utils
from django_admin_otp.forms import OTPForm
from django_admin_otp.models import OTPVerification, TrustedDevice


def _mfa_verify_success_response(request, user, trust_device):
    request.session[settings.MFA_VERIFIED_SESSION_KEY] = True
    response = redirect(utils.admin_url())

    if trust_device:
        device = TrustedDevice.create_for_user(user=user, device_info=request.META["HTTP_USER_AGENT"])
        response.set_cookie(
            key=settings.DEVICE_TOKEN_COOKIE_NAME,
            value=device.token_cipher,
            max_age=settings.TRUSTED_DEVICE_DAYS * 24 * 60 * 60,  # seconds
            httponly=True,
            secure=not global_settings.DEBUG,  # secured in production
            samesite="Lax",
        )
    return response


def _mfa_verify_get_response(request):
    if utils.is_request_mfa_verified(request) or utils.is_trusted_device_request(request):
        return redirect(utils.admin_url())

    if not OTPVerification.objects.filter(user=request.user, confirmed=True).exists():
        if settings.FORCE_OTP:
            return redirect(settings.MFA_SETUP_INTERNAL_NAME)
        return redirect(utils.admin_url())

    return render(request, "mfa_verify.html")


@login_required
def mfa_verify(request):
    if request.method != "POST":
        return _mfa_verify_get_response(request)

    user = request.user
    form = OTPForm(request.POST)
    if not form.is_valid():
        return render(request, "mfa_verify.html", {"error": "Wrong form data"})

    verification = OTPVerification.objects.only("secret_key_cipher").get(user=user, confirmed=True)
    if verification.verify(form.cleaned_data["code"]):
        return _mfa_verify_success_response(
            request=request,
            user=user,
            trust_device=form.cleaned_data["trust_device"],
        )

    return render(request, "mfa_verify.html", {"error": "Wrong code"})


def _mfa_setup_get_response(request, verification):
    if verification.confirmed:
        # if user already has the confirmed verification - skip it
        return redirect(utils.admin_url())

    return render(
        request,
        "mfa_setup.html",
        {"qr_code_url": utils.generate_qr_image(verification.generate_qr_code_uri())},
    )


@login_required
def mfa_setup(request):
    verification, _ = OTPVerification.objects.get_or_create(user=request.user)
    if request.method != "POST":
        return _mfa_setup_get_response(request, verification)

    form = OTPForm(request.POST)
    if not form.is_valid():
        return render(request, "mfa_setup.html", {"error": "Wrong form data"})

    if verification.verify(form.cleaned_data["code"]):
        verification.confirmed = True
        verification.save()
        return redirect(utils.admin_url())

    return render(
        request,
        "mfa_setup.html",
        {
            "qr_code_url": utils.generate_qr_image(verification.generate_qr_code_uri()),
            "error": "Wrong code",
        },
    )



import base64
import io

import qrcode

from django_admin_otp import settings
from django_admin_otp.models import TrustedDevice


def generate_qr_image(uri):
    """Returns base64-image for QR-code"""
    qr = qrcode.make(uri)
    buf = io.BytesIO()
    qr.save(buf, format="PNG")
    img_b64 = base64.b64encode(buf.getvalue()).decode("utf-8")
    return f"data:image/png;base64,{img_b64}"


def admin_url():
    return f"/{settings.ADMIN_PATH}"


def is_trusted_device_request(request):
    trusted_token_cipher = request.COOKIES.get(settings.DEVICE_TOKEN_COOKIE_NAME)
    if not trusted_token_cipher:
        return False

    # If user has trusted device cookie and device is exists - no need to check
    return (
        TrustedDevice.objects.filter(user=request.user)
        .by_token_cipher(token_cipher=trusted_token_cipher)
        .active()
        .exists()
    )


def is_request_mfa_verified(request):
    return bool(request.session.get(settings.MFA_VERIFIED_SESSION_KEY, False))



from django.shortcuts import redirect
from django.urls import reverse

from django_admin_otp import settings
from django_admin_otp.models import OTPVerification
from django_admin_otp.utils import admin_url, is_request_mfa_verified, is_trusted_device_request


class AdminOTPMiddleware:
    """Middleware for detecting if MFA is required for a request"""

    def __init__(self, get_response):
        self.get_response = get_response

        self._admin_prefix = admin_url()

    def _is_verify_needed(self, request):
        # MFA is only checked for the admin site and authenticated users
        if not (request.path.startswith(self._admin_prefix) and request.user.is_authenticated):
            return False

        # If MFA is already verified in the session → no need to check
        return not is_request_mfa_verified(request)

    def __call__(self, request):
        if not self._is_verify_needed(request):
            return self.get_response(request)

        if not OTPVerification.objects.filter(user=request.user, confirmed=True).exists():
            if settings.FORCE_OTP and request.path != reverse(settings.MFA_SETUP_INTERNAL_NAME):
                return redirect(settings.MFA_SETUP_INTERNAL_NAME)
            return self.get_response(request)

        if is_trusted_device_request(request):
            return self.get_response(request)

        # Ignore MFA check path to avoid redirect loops
        if request.path != reverse(settings.MFA_VERIFY_INTERNAL_NAME):
            return redirect(settings.MFA_VERIFY_INTERNAL_NAME)

        return self.get_response(request)



from django_admin_otp import settings as configuration


def settings(request):  # noqa: ARG001
    return {
        "FORCE_OTP": configuration.FORCE_OTP,
        "TRUSTED_DEVICE_DAYS": configuration.TRUSTED_DEVICE_DAYS,
    }



from datetime import timedelta

import pyotp
from django.contrib.auth import get_user_model
from django.core import signing
from django.db import models
from django.utils import timezone
from django.utils.crypto import get_random_string

from django_admin_otp import settings


def generate_secret_key_cipher():
    return signing.dumps(pyotp.random_base32())


User = get_user_model()


class OTPVerification(models.Model):
    user = models.OneToOneField(
        User,
        on_delete=models.CASCADE,
        related_name="admin_otp_verification",
        verbose_name="user",
    )
    secret_key_cipher = models.CharField(max_length=255, default=generate_secret_key_cipher)
    confirmed = models.BooleanField(default=False)
    created_at = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return f"OTP Verification for User {self.user_id}"

    @property
    def secret_key(self):
        return signing.loads(self.secret_key_cipher)

    @property
    def totp(self):
        return pyotp.TOTP(self.secret_key)

    def generate_qr_code_uri(self):
        return self.totp.provisioning_uri(
            name=self.user.username,
            issuer_name=settings.PROJECT_NAME,
        )

    def verify(self, code):
        return self.totp.verify(code)


class TrustedDeviceQuerySet(models.QuerySet):
    def active(self):
        return self.filter(expires_at__gt=timezone.now())

    def by_token_cipher(self, token_cipher):
        return self.filter(token=signing.loads(token_cipher))


class TrustedDevice(models.Model):
    user = models.ForeignKey(User, on_delete=models.CASCADE, related_name="admin_otp_trusted_devices")
    device_info = models.CharField(max_length=255)
    token = models.CharField(max_length=64, unique=True)
    created_at = models.DateTimeField(auto_now_add=True)
    expires_at = models.DateTimeField()

    objects = TrustedDeviceQuerySet.as_manager()

    def __str__(self):
        return f"OTP Trusted Device for User {self.user_id}"

    @classmethod
    def create_for_user(cls, user, device_info):
        token = get_random_string(64)
        expires_at = timezone.now() + timedelta(days=settings.TRUSTED_DEVICE_DAYS)
        return cls.objects.create(
            user=user,
            token=token,
            device_info=device_info,
            expires_at=expires_at,
        )

    @property
    def token_cipher(self):
        return signing.dumps(self.token)



from django import forms
from django.contrib import admin, messages
from django.contrib.auth import get_user_model, logout
from django.db.transaction import atomic
from django.shortcuts import redirect, render
from django.urls import path

from django_admin_otp import settings, utils
from .models import OTPVerification, TrustedDevice

User = get_user_model()


class MFAForm(forms.Form):
    code = forms.CharField(
        max_length=6,
        required=True,
        widget=forms.TextInput(
            attrs={
                "maxlength": 6,
                "pattern": "[0-9]*",
                "inputmode": "numeric",
            },
        ),
        label="MFA Code",
    )


@admin.register(OTPVerification)
class OTPVerificationAdmin(admin.ModelAdmin):
    list_display = ("user", "confirmed", "created_at")
    readonly_fields = ("user", "confirmed", "created_at")
    actions = None  # remove standart actions
    fields = (
        "user",
        "confirmed",
        "created_at",
    )
    change_list_template = "admin/otpverification_changelist.html"

    def has_add_permission(self, request):
        return False

    def has_change_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def get_urls(self):
        urls = super().get_urls()
        custom_urls = [
            path("setup-mfa/", self.admin_site.admin_view(self.setup_mfa), name="setup-mfa"),
            path("disable-mfa/", self.admin_site.admin_view(self.disable_mfa), name="disable-mfa"),
        ]
        return custom_urls + urls

    def qr_preview(self, obj):
        if not obj or obj.confirmed:
            return ""
        uri = obj.generate_qr_code_uri()
        img_data = utils.generate_qr_image(uri)
        return f'<img src="{img_data}" width="200" height="200"/>'

    qr_preview.allow_tags = True
    qr_preview.short_description = "QR Code"

    @atomic
    def setup_mfa(self, request):
        user = request.user
        verification, _ = OTPVerification.objects.get_or_create(user=user)

        if verification.confirmed:
            messages.warning(request, "MFA has been already connected")
            return redirect("..")

        form = MFAForm()
        if request.method == "POST":
            form = MFAForm(request.POST)
            if form.is_valid():
                code = form.cleaned_data["code"]
                if verification.verify(code):
                    verification.confirmed = True
                    verification.save()
                    messages.success(request, "MFA had been connected successfully")
                    logout(request)
                    return redirect("..")

                form.add_error("code", "Wrong code")

        uri = verification.generate_qr_code_uri()
        qr_img = utils.generate_qr_image(uri)
        return render(
            request,
            "admin/mfa_popup.html",
            {
                "form": form,
                "qr_img": qr_img,
                "title": "Connect MFA",
            },
        )

    @atomic
    def disable_mfa(self, request):
        user = request.user
        verification = getattr(user, "admin_otp_verification", None)
        if not verification or not verification.confirmed:
            messages.warning(request, "MFA hasn't been connected")
            return redirect("..")

        form = MFAForm()
        if request.method == "POST":
            form = MFAForm(request.POST)
            if form.is_valid():
                code = form.cleaned_data["code"]
                if verification.verify(code):
                    self._disable_cleanup(request, verification, user)
                    logout(request)
                    messages.success(request, "MFA has been disconnected")
                    response = redirect("..")
                    response.delete_cookie(settings.DEVICE_TOKEN_COOKIE_NAME)
                    return response

                form.add_error("code", "Wrong code")

        return render(request, "admin/mfa_popup.html", {"form": form, "title": "Disable MFA"})

    def _disable_cleanup(self, request, verification, user):
        verification.delete()
        TrustedDevice.objects.filter(user=user).delete()
        if settings.MFA_VERIFIED_SESSION_KEY in request.session:
            del request.session[settings.MFA_VERIFIED_SESSION_KEY]


@admin.register(TrustedDevice)
class TrustedDeviceAdmin(admin.ModelAdmin):
    list_display = ("id", "user", "device_info", "created_at", "expires_at")
    list_filter = ["expires_at", "created_at"]
    search_fields = ["user__username", "user__id", "device_info__icontains"]
    autocomplete_fields = ["user"]
    readonly_fields = ["user"]
    exclude = ["token"]

    def has_change_permission(self, request, obj=None):
        if not obj:
            return False

        return obj.user == request.user

    def has_delete_permission(self, request, obj=None):
        if not obj:
            return False

        return obj.user == request.user



from django.contrib.auth.views import LogoutView
from django.urls import path

from django_admin_otp import settings, utils
from .views import mfa_setup, mfa_verify

urlpatterns = [
    path("verify/", mfa_verify, name=settings.MFA_VERIFY_INTERNAL_NAME),
    path("setup/", mfa_setup, name=settings.MFA_SETUP_INTERNAL_NAME),
    path("logout/", LogoutView.as_view(next_page=utils.admin_url()), name="admin-otp-logout"),
]



from datetime import timedelta
from unittest import mock
from unittest.mock import patch

from django.contrib import admin
from django.contrib.auth.models import AnonymousUser, User
from django.core import signing
from django.test import Client, RequestFactory, TestCase
from django.urls import reverse
from django.utils import timezone
from django.utils.crypto import get_random_string

from django_admin_otp import settings
from django_admin_otp.admin import MFAForm, TrustedDeviceAdmin
from django_admin_otp.middleware import AdminOTPMiddleware
from django_admin_otp.models import OTPVerification, TrustedDevice
from django_admin_otp.utils import admin_url
from django_admin_otp.views import mfa_setup, mfa_verify


class AdminOTPMiddlewareTest(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.user = User.objects.create_user(username="testuser", password=get_random_string(16), is_staff=True)
        self.middleware = AdminOTPMiddleware(get_response=lambda _: "OK")
        self.admin_path = f"/{settings.ADMIN_PATH}/some-page/"
        self.verify_url = reverse(settings.MFA_VERIFY_INTERNAL_NAME)
        self.setup_url = reverse(settings.MFA_SETUP_INTERNAL_NAME)

    def test_unauthenticated_user_passes(self):
        request = self.factory.get(self.admin_path)
        request.user = AnonymousUser()

        response = self.middleware(request)

        self.assertEqual(response, "OK")

    def test_verified_in_session_passes(self):
        request = self.factory.get(self.admin_path)
        request.user = self.user
        request.session = {settings.MFA_VERIFIED_SESSION_KEY: True}

        response = self.middleware(request)

        self.assertEqual(response, "OK")

    def test_verified_in_session_no_trusted_device_no_force_otp(self):
        request = self.factory.get(self.admin_path)
        request.user = self.user
        request.session = {}
        request.COOKIES = {}

        response = self.middleware(request)

        self.assertEqual(response, "OK")

    def test_requires_mfa_redirects_to_verify(self):
        request = self.factory.get(self.admin_path)
        OTPVerification.objects.create(user=self.user, confirmed=True, secret_key_cipher=signing.dumps("abc"))
        request.user = self.user
        request.session = {}
        request.COOKIES = {}

        response = self.middleware(request)

        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, self.verify_url)

    def test_trusted_device_allows_access(self):
        OTPVerification.objects.create(user=self.user, confirmed=True, secret_key_cipher=signing.dumps("abc"))
        device = TrustedDevice.create_for_user(user=self.user, device_info="test-agent")
        request = self.factory.get(self.admin_path)
        request.user = self.user
        request.session = {}
        request.COOKIES = {settings.DEVICE_TOKEN_COOKIE_NAME: device.token_cipher}

        response = self.middleware(request)

        self.assertEqual(response, "OK")

    def test_force_otp_redirects_to_setup(self):
        old_value = settings.FORCE_OTP
        settings.FORCE_OTP = True
        try:
            request = self.factory.get(self.admin_path)
            request.user = self.user
            request.session = {}
            request.COOKIES = {}

            response = self.middleware(request)

            self.assertEqual(response.status_code, 302)
            self.assertEqual(response.url, self.setup_url)
        finally:
            settings.FORCE_OTP = old_value

    def test_access_to_mfa_verify_page_does_not_redirect(self):
        OTPVerification.objects.create(user=self.user, confirmed=True, secret_key_cipher=signing.dumps("abc"))
        request = self.factory.get(self.verify_url)
        request.user = self.user
        request.session = {}
        request.COOKIES = {}

        response = self.middleware(request)

        self.assertEqual(response, "OK")

    def test_trusted_device_inactive_requires_mfa(self):
        OTPVerification.objects.create(user=self.user, confirmed=True, secret_key_cipher=signing.dumps("abc"))
        device = TrustedDevice.create_for_user(
            user=self.user,
            device_info="test-agent",
        )
        device.expires_at = timezone.now() - timedelta(days=1)
        device.save()

        request = self.factory.get(self.admin_path)
        request.user = self.user
        request.session = {}
        request.COOKIES = {settings.DEVICE_TOKEN_COOKIE_NAME: device.token_cipher}

        response = self.middleware(request)

        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, self.verify_url)

    def test_request_outside_admin_prefix_passes(self):
        OTPVerification.objects.create(user=self.user, confirmed=True, secret_key_cipher=signing.dumps("abc"))
        path = "/some-other-page/"
        request = self.factory.get(path)
        request.user = self.user
        request.session = {}
        request.COOKIES = {}

        response = self.middleware(request)

        self.assertEqual(response, "OK")

    def test_no_otp_force_otp_false_passes(self):
        old_value = settings.FORCE_OTP
        settings.FORCE_OTP = False
        try:
            request = self.factory.get(self.admin_path)
            request.user = self.user
            request.session = {}
            request.COOKIES = {}

            response = self.middleware(request)

            self.assertEqual(response, "OK")
        finally:
            settings.FORCE_OTP = old_value


class MFAVerifyViewTest(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.user = User.objects.create_user(username="testuser", password="12345", is_staff=True)
        self.verification = OTPVerification.objects.create(user=self.user, confirmed=True, secret_key_cipher="abc")

    @patch.object(OTPVerification, "verify", return_value=True)
    def test_post_valid_code_trust_device(self, mock_verify):
        request = self.factory.post("/mfa-verify/", data={"code": "123456", "trust_device": True})
        request.user = self.user
        request.session = {}
        request.META["HTTP_USER_AGENT"] = "test-agent"

        response = mfa_verify(request)

        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, admin_url())
        self.assertIn(settings.DEVICE_TOKEN_COOKIE_NAME, response.cookies)
        self.assertTrue(request.session.get(settings.MFA_VERIFIED_SESSION_KEY))

    @patch.object(OTPVerification, "verify", return_value=True)
    def test_post_valid_code_no_trust_device(self, mock_verify):
        request = self.factory.post("/mfa-verify/", data={"code": "123456", "trust_device": False})
        request.user = self.user
        request.session = {}
        request.META["HTTP_USER_AGENT"] = "test-agent"

        response = mfa_verify(request)

        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, admin_url())
        self.assertNotIn(settings.DEVICE_TOKEN_COOKIE_NAME, response.cookies)
        self.assertTrue(request.session.get(settings.MFA_VERIFIED_SESSION_KEY))

    def test_post_invalid_form(self):
        request = self.factory.post("/mfa-verify/", data={"code": "9999990", "trust_device": True})
        request.user = self.user
        request.session = {}
        request.META["HTTP_USER_AGENT"] = "test-agent"

        response = mfa_verify(request)

        self.assertEqual(response.status_code, 200)
        self.assertIn("Wrong form", response.content.decode())

    def test_post_verify_error(self):
        request = self.factory.post("/mfa-verify/", data={"code": "9999990", "trust_device": True})
        request.user = self.user
        request.session = {}
        request.META["HTTP_USER_AGENT"] = "test-agent"

        with patch.object(OTPVerification, "verify", return_value=False):
            response = mfa_verify(request)

        self.assertEqual(response.status_code, 200)
        self.assertIn("Wrong form", response.content.decode())

    def test_get_request_renders_template(self):
        request = self.factory.get("/mfa-verify/")
        request.user = self.user
        request.session = {}

        response = mfa_verify(request)

        self.assertEqual(response.status_code, 200)

    def test_get_request_go_to_admin_cause_already_verified(self):
        request = self.factory.get("/mfa-verify/")
        request.user = self.user
        request.session = {settings.MFA_VERIFIED_SESSION_KEY: True}

        response = mfa_verify(request)

        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, admin_url())

    def test_get_request_go_to_admin_cause_trusted_device(self):
        request = self.factory.get("/mfa-verify/")
        device = TrustedDevice.create_for_user(self.user, device_info="test")
        request.user = self.user
        request.session = {}
        request.COOKIES = {settings.DEVICE_TOKEN_COOKIE_NAME: device.token_cipher}

        response = mfa_verify(request)

        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, admin_url())

    def test_get_request_go_to_admin_cause_no_verification_force_otp(self):
        old_value = settings.FORCE_OTP
        settings.FORCE_OTP = 1
        try:
            self.verification.delete()
            request = self.factory.get("/mfa-verify/")
            request.user = self.user
            request.session = {}
            request.COOKIES = {}

            response = mfa_verify(request)

            self.assertEqual(response.status_code, 302)
            self.assertEqual(response.url, reverse(settings.MFA_SETUP_INTERNAL_NAME))
        finally:
            settings.FORCE_OTP = old_value

    def test_get_request_go_to_admin_cause_no_verification_no_force_otp(self):
        old_value = settings.FORCE_OTP
        settings.FORCE_OTP = 0
        try:
            self.verification.delete()
            request = self.factory.get("/mfa-verify/")
            request.user = self.user
            request.session = {}
            request.COOKIES = {}

            response = mfa_verify(request)

            self.assertEqual(response.status_code, 302)
            self.assertEqual(response.url, admin_url())
        finally:
            settings.FORCE_OTP = old_value


class MFASetupViewTest(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.user = User.objects.create_user(username="setupuser", password="12345", is_staff=True)

    @patch("django_admin_otp.utils.generate_qr_image", return_value="qr_image")
    def test_get_renders_qr(self, mock_qr):
        request = self.factory.get("/mfa-setup/")
        request.user = self.user
        request.session = {}

        response = mfa_setup(request)

        self.assertEqual(response.status_code, 200)
        self.assertIn("qr_image", response.content.decode())

    @patch("django_admin_otp.utils.generate_qr_image", return_value="qr_image")
    def test_get_confirmed_verification_already_exists_but_not_confirmed(self, mock_qr):
        request = self.factory.get("/mfa-setup/")
        OTPVerification.objects.create(user=self.user, confirmed=False)
        request.user = self.user
        request.session = {}

        response = mfa_setup(request)

        self.assertEqual(response.status_code, 200)
        self.assertIn("qr_image", response.content.decode())

    @patch("django_admin_otp.utils.generate_qr_image", return_value="qr_image")
    def test_get_confirmed_verification_already_exists_and_confirmed(self, mock_qr):
        request = self.factory.get("/mfa-setup/")
        OTPVerification.objects.create(user=self.user, confirmed=True)
        request.user = self.user
        request.session = {}

        response = mfa_setup(request)

        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, admin_url())

    @patch.object(OTPVerification, "verify", return_value=True)
    @patch("django_admin_otp.views.utils.generate_qr_image", return_value="qr_image")
    def test_post_valid_code_confirms_otp(self, mock_qr, mock_verify):
        verification = OTPVerification.objects.create(user=self.user)
        request = self.factory.post("/mfa-setup/", data={"code": "123456"})
        request.user = self.user
        request.session = {}

        response = mfa_setup(request)
        verification.refresh_from_db()

        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, admin_url())
        self.assertTrue(verification.confirmed)

    @patch.object(OTPVerification, "verify", return_value=False)
    @patch("django_admin_otp.views.utils.generate_qr_image", return_value="qr_image")
    def test_post_invalid_code_renders_template_wrong_form(self, mock_qr, mock_verify):
        OTPVerification.objects.create(user=self.user)
        request = self.factory.post("/mfa-setup/", data={"code": "9999990"})
        request.user = self.user
        request.session = {}

        response = mfa_setup(request)

        self.assertEqual(response.status_code, 200)
        self.assertIn("Wrong form data", response.content.decode())

    @patch.object(OTPVerification, "verify", return_value=False)
    @patch("django_admin_otp.views.utils.generate_qr_image", return_value="qr_image")
    def test_post_invalid_code_renders_template_invalid_code(self, mock_qr, mock_verify):
        verification = OTPVerification.objects.create(user=self.user)
        verification.verify = mock.Mock(return_value=False)
        request = self.factory.post("/mfa-setup/", data={"code": "999999"})
        request.user = self.user
        request.session = {}

        response = mfa_setup(request)

        self.assertEqual(response.status_code, 200)
        self.assertIn("Wrong code", response.content.decode())


class OTPVerificationAdminTest(TestCase):
    def setUp(self):
        self.client = Client()
        self.user = User.objects.create_superuser(username="admin", password="12345", email="admin@test.com")
        self.client.login(username="admin", password="12345")
        self.verification = OTPVerification.objects.create(
            user=self.user,
            confirmed=False,
            secret_key_cipher=signing.dumps("abc"),
        )
        settings.FORCE_OTP = False

    @patch("django_admin_otp.utils.generate_qr_image", return_value="qr_image")
    def test_setup_mfa_get_renders_template(self, mock_qr):
        url = reverse("admin:setup-mfa")

        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "qr_image")
        self.assertIsInstance(response.context["form"], MFAForm)

    @patch.object(OTPVerification, "verify", return_value=True)
    def test_setup_mfa_post_valid_code_confirms(self, mock_verify):
        url = reverse("admin:setup-mfa")

        response = self.client.post(url, data={"code": "123456"})
        self.verification.refresh_from_db()

        self.assertEqual(response.status_code, 302)
        self.assertTrue(self.verification.confirmed)
        self.assertFalse(response.wsgi_request.user.is_authenticated)  # logout after successful setup mfa

    @patch.object(OTPVerification, "verify", return_value=False)
    def test_setup_mfa_post_invalid_code_shows_error(self, mock_verify):
        url = reverse("admin:setup-mfa")

        response = self.client.post(url, data={"code": "wrong"})

        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "Wrong code")

    @patch.object(OTPVerification, "verify", return_value=True)
    def test_disable_mfa_post_valid_code_deletes_verification(self, mock_verify):
        self.verification.confirmed = True
        self.verification.save()
        TrustedDevice.create_for_user(self.user, device_info="test")
        session = self.client.session
        session[settings.MFA_VERIFIED_SESSION_KEY] = True
        session.save()
        url = reverse("admin:disable-mfa")

        response = self.client.post(url, data={"code": "123456"})

        self.assertEqual(response.status_code, 302)
        self.assertFalse(OTPVerification.objects.filter(user=self.user).exists())
        self.assertFalse(TrustedDevice.objects.filter(user=self.user).exists())
        self.assertFalse(response.wsgi_request.user.is_authenticated)
        self.assertTrue(settings.DEVICE_TOKEN_COOKIE_NAME in response.cookies)

    @patch.object(OTPVerification, "verify", return_value=False)
    def test_disable_mfa_post_invalid_code_shows_error(self, mock_verify):
        self.verification.confirmed = True
        self.verification.save()
        session = self.client.session
        session[settings.MFA_VERIFIED_SESSION_KEY] = True
        session.save()
        url = reverse("admin:disable-mfa")

        response = self.client.post(url, data={"code": "wrong"})

        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "Wrong code")


class TrustedDeviceAdminTest(TestCase):
    def setUp(self):
        self.client = Client()
        self.user = User.objects.create_superuser(username="admin", password="12345", email="admin@test.com")
        self.client.login(username="admin", password="12345")
        self.device = TrustedDevice.create_for_user(user=self.user, device_info="test-device")

    def test_change_permission_for_owner(self):
        admin_instance = TrustedDeviceAdmin(TrustedDevice, admin.site)

        self.assertTrue(
            admin_instance.has_change_permission(request=self.client.request().wsgi_request, obj=self.device),
        )

    def test_change_permission_for_other_user(self):
        other_user = User.objects.create_user(username="other", password="12345")
        self.device.user = other_user
        self.device.save()

        admin_instance = TrustedDeviceAdmin(TrustedDevice, admin.site)

        self.assertFalse(
            admin_instance.has_change_permission(request=self.client.request().wsgi_request, obj=self.device),
        )

    def test_delete_permission_for_owner(self):
        admin_instance = TrustedDeviceAdmin(TrustedDevice, admin.site)

        self.assertTrue(
            admin_instance.has_delete_permission(request=self.client.request().wsgi_request, obj=self.device),
        )

    def test_delete_permission_for_other_user(self):
        other_user = User.objects.create_user(username="other", password="12345")
        self.device.user = other_user
        self.device.save()

        admin_instance = TrustedDeviceAdmin(TrustedDevice, admin.site)

        self.assertFalse(
            admin_instance.has_delete_permission(request=self.client.request().wsgi_request, obj=self.device),
        )



import os

from django.conf import settings

MFA_VERIFIED_SESSION_KEY = "mfa-verified"
MFA_VERIFY_INTERNAL_NAME = "mfa-verify"
MFA_SETUP_INTERNAL_NAME = "mfa-setup"

PROJECT_NAME = os.environ.get("ADMIN_OTP_PROJECT_NAME", None)
DEVICE_TOKEN_COOKIE_NAME = os.environ.get("ADMIN_OTP_DEVICE_TOKEN_COOKIE_NAME", "admin_otp_trusted_device")
ADMIN_PATH = os.environ.get("ADMIN_PATH", "admin/")
TRUSTED_DEVICE_DAYS = os.environ.get("ADMIN_OTP_TRUSTED_DEVICE_DAYS", 30)
FORCE_OTP = bool(int(os.environ.get("ADMIN_OTP_FORCE", 0)))


def init():
    if not settings.TEMPLATES:
        settings.TEMPLATES = [
            {
                "BACKEND": "django.template.backends.django.DjangoTemplates",
                "DIRS": [],
                "APP_DIRS": True,
                "OPTIONS": {
                    "context_processors": [
                        "django.template.context_processors.request",
                        "django.contrib.auth.context_processors.auth",
                        "django.contrib.messages.context_processors.messages",
                        "django_admin_otp.context_processors.admin_otp.settings",
                    ],
                },
            },
        ]

    settings.TEMPLATES[0]["OPTIONS"]["context_processors"].append(
        "django_admin_otp.context_processors.admin_otp.settings",
    )



# Generated by Django 5.2.5 on 2025-08-28 08:34

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models

import django_admin_otp.models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="OTPVerification",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "secret_key_cipher",
                    models.CharField(default=django_admin_otp.models.generate_secret_key_cipher, max_length=255),
                ),
                ("confirmed", models.BooleanField(default=False)),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                (
                    "user",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="admin_otp_verification",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="user",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="TrustedDevice",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("device_info", models.CharField(max_length=255)),
                ("token", models.CharField(max_length=64, unique=True)),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("expires_at", models.DateTimeField()),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="admin_otp_trusted_devices",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
    ]
